#include "config.h"
#include <string.h>
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cassert>
#include <cstdlib>
#include <climits>
#include <cmath>
#include "subprob.h"
using namespace std;
//File below included for time (general) routines

#include "general.h" 
//Apply below to keep the states as a lists of profits, each profit having its own list of costs
//Disable below to simply keep the states as unique lists of (profit, cost).
#define LISTOFPROFITS

//capacity multiplied by 1000, each weight becomes 1000weight-i%10. Each un-scaled solution remains
//valid. Any configuration unfeasible in the non-scaled versions is still unfeasible.
//#define SCALED_INSTANCE

//Define below if you do *not* want to solve the knapsack problem (bound for return route) using
//dynamical programming. This makes sense when the capacity is >1000000. Instead of solving the
//knapsack problem, an upper bound for it is obtained by sorting the items according to their
//profits and by filling the knapsack in this order, no 1..C loop. Better algorithms do exist!
//#define NO_DP_KP_BND

namespace problemSpecific{
/*================================     INPUT/OUTPUT ROUTINES     ==================================*/
fstream    inStream;                         //input stream
char*      inputFile=NULL;                   //Using char* for compatibility with fstream
int        currInst;                         //the instance number in the input file

int        vtxNo;                            //nr of vertices
int        n;                                //n required edges
int        m;                                //total number of edges (including non requiring serv)
double     C;                                //capacity
int        dep;                              //depot index
double*    lb;                               //the same as ::l, lower bound on each variable

typedef struct edge_{
    int    i  ;                              //index in the [0..n-1] solutions
    int    r  ;                              //the profit r
    int    vtx1;
    int    vtx2;
    double w;
    double b;
    double dc;                               //deadheading cost
    struct edge_* nxt;
}edge;

edge**  g=NULL;                              //input graph; a vector of pointers
double **  costDiff;                         //minimum cost for all pairs source/destination


//This function returns 1 if the file ends in .txt, 0 otherwise
int checkTxtFormat(char*cspFile){
    int pointPos = strlen(cspFile)-4;
    if(cspFile[pointPos]!='.'||//no extension
      (cspFile[pointPos+1]=='t'&&cspFile[pointPos+2]=='x'&&cspFile[pointPos+3]=='t')//.txt
    ){
            clog<<"Reading a classical file dat file (no 3-character extension=.txt):"<<cspFile<<".\n";
            return 1;
    }
    cerr<<"Unknown file format (extension '"<<cspFile+pointPos+1<<"') not recognized\n"; 
    return 0;
}
void   buildCostDiff();                      //fwd declarations for use in function below
double getVarBound(int i, int v1, int v2, double weight);            // bound on variable i

//Return 1 if successful, or 0 if error
int readNextInst(int &nGlb, double* &l, double* &u, double* &b, double &lBndBase){
    
    string currLine("c");                              
    while(inStream.good()                    //jump comments line before or empty end of lines
          &&(currLine[0]=='c'||currLine.find_first_not_of(" \t\n\v\f\r") == string::npos)){
        getline(inStream,currLine);
        clog<<" st comment line___"<<currLine<<"___end comment\n";
    }
    vtxNo = atoi(currLine.c_str());          //nr of vertices is in the last read line above

    inStream >> n;                           //required edges
    if(!inStream.good())                     //test again EOF, as getline(..) above might no set eof
        return 0;

    nGlb = n;
    inStream >> m;                           //total number of edges
    m += n;                 
    inStream >> C;
    cout<<"Currinst="<<setw(3)<<currInst;
    #ifdef SCALED_INSTANCE
        cout<<"$^*$";
        C = C * 1000;
    #endif
    #ifndef NO_DP_KP_BND
      if(C>1000000)
        cerr<<"The capacity seems quite large. Think to enable NO_DP_KP_BND to be even safer\n";
    #endif
    cout<<setw(3)<<n<<"&"<<setw(3)<<vtxNo;

    b      = new double[n];
    u      = new double[n];
    l      = new double[n];
    costDiff = new double*[vtxNo];
    g      = new edge*[vtxNo];
    for(int v=0;v<vtxNo;v++){
        g[v] = NULL;
        costDiff[v] = new double[vtxNo];
        for(int u=0;u<vtxNo;u++)
            costDiff[v][u] = INT_MAX;
    }
    int * endVtx1Loc   = new int[n];         //end vertex 1 for each requested edge (local var)
    int * endVtx2Loc   = new int[n];         //end vertex 2 (local var)
    double * weightLoc = new double[n];      //weight for each required edge

    for(int i=0;i<m;i++){ 
        int v1, v2;
        edge * next1, * next2;
        inStream >> v1;
        inStream >> v2;
        if(v1==v2){
            cerr<<"I do not yet accept reflexive edges (v,v): ignoring: "<<v1<<","<<v2<<endl;
            continue;
        }
        v1--;
        v2--;
        next1         = g[v1];
        next2         = g[v2];
        g[v1]         = new edge;
        g[v2]         = new edge;
        g[v1]->nxt    = next1;
        g[v2]->nxt    = next2;
        g[v1]->vtx1   = v1;
        g[v1]->vtx2   = v2;
        g[v2]->vtx1   = v2;
        g[v2]->vtx2   = v1;
        g[v1]->i      = i; 
        g[v2]->i      = i; 
        g[v1]->b      = 0;                       //no demand by default
        g[v2]->b      = 0;              
        g[v1]->r      = 0;                       //no profit by default
        g[v2]->r      = 0;              
        g[v1]->w      = INT_MAX/2;               //prohibitive weight by default, not to be used
        g[v2]->w      = INT_MAX/2;               //unless it is a requested edge (w modified below)

        inStream >> g[v1]->dc;
        g[v2]->dc = g[v1]->dc;

        if(i<n){                                 //required edges
            g[v1]->b      = 1;
            g[v2]->b      = 1;
            b[i]          = g[v1]-> b;

            inStream >> g[v1]->w;
            #ifdef SCALED_INSTANCE
                 g[v1]->w = g[v1]->w * 1000 - i%10;
            #endif
            g[v2]->w = g[v1]->w;

            l[i] = -g[v1]->dc;
            u[i] = INT_MAX;

            endVtx1Loc[i] = v1;
            endVtx2Loc[i] = v2;
            weightLoc[i]  =  g[v1]->w;
        }
    }
    inStream>>dep;
    dep--;
    currInst++;
    cout<<"&";
    
    lb = l;
    buildCostDiff();
    lBndBase = 0;
    for(int i=0;i<n;i++){
        u[i] = getVarBound(i,endVtx1Loc[i],endVtx2Loc[i], weightLoc[i]);
        lBndBase-=(b[i]*l[i]);
    }
    clog<<"startObj start Objective="<<lBndBase<<endl;
    delete[] endVtx1Loc; 
    delete[] endVtx2Loc; 
    delete[] weightLoc;
    return 1;
}

//Prepare the data to be able to load a new input file 
//Input : filename
//Return: stream inStream, int currInst, int instanceType, int instNo (nr of instance)
//In can exit program with error if the instance type is not (yet) recognized
void prepareNewInput(char*filename){
    if(inputFile!=NULL)
        delete[] inputFile;

    inputFile = new char[strlen(filename)+1];
    strcpy(inputFile,filename);
    inStream.open(inputFile, fstream::in);  
    currInst = 1;                                //first instance to be loaded 
    if(!checkTxtFormat(inputFile))
        exit(EXIT_FAILURE);
}
//return value: 0 when there is nothing more to load, 1 otherwise
//return implicit: number of variables n,
//                 lower bound on each variable l
//                 objective function coefficients b
//Other problem specific data is loaded in: problemSpecific::variables
int loadData(char *filename, int &n, double* &l, double* &u, double* &b, double & lBndBase){
    if(inputFile==NULL||                         //first call ever
       strcmp(inputFile,filename) )              //the input file changed
            prepareNewInput(filename);
    if(!inStream.good()){
        clog<<"No more instances\n";
        inStream.close();
        return 0;
    }

    if(b!=NULL){                                  //erase memory if an instance existed befor
        delete[] l;
        delete[] u;
        delete[] b;
        for(int i=0;i<vtxNo;i++){
            while (g[i]!=NULL){
                edge * next = g[i]->nxt;
                delete g[i];
                g[i] = next;
            }
            delete[] costDiff[i];
        }
        delete[] g;
        delete[] costDiff;
    }
    l=NULL;b=NULL;g=NULL;
         
    if(!readNextInst(n,l,u,b,lBndBase)){           //try to read more, but stop if impossible
        clog<<"No more instances could be read\n";
        inStream.close();
        return 0;
    }

    return 1;                                  
}
//by defining NO_DP_KP_BND, we are safe against capacities larger than 1000000
#ifdef NO_DP_KP_BND
  typedef struct effList_{
      double w;
      double p;
      struct effList_ * nxt;
  } effList;
  
  effList*efl;
  void eflMerge(double w, double p){
      effList* it1, *it2;
      it1 = efl;
      it2 = efl;
      while ( (it1!=NULL)&&(it1->p/it1->w > p/w) ){
          it2 = it1;
          it1 = it1->nxt;
      }
      if(it2==efl){
          efl = new effList;
          efl->w = w;
          efl->p = p;
          efl->nxt = it1;
          return;
      }
      it2->nxt = new effList;
      it2->nxt->w = w;
      it2->nxt->p = p;
      it2->nxt->nxt = it1;
  }
#else
  double* mostEffRet;                               //most effective (max prof) using any weight<=C!
#endif

//Copy the table r into the graph adjacency list structure
void rCpyToGraph(int*r){
    //by defining NO_DP_KP_BND, we are safe against capacities larger than 1000000
    #ifdef NO_DP_KP_BND
        efl = NULL;
    #else
        mostEffRet = new double[(int)C+1];
        for(int ii=0;ii<=C;ii++) 
            mostEffRet[ii]=0;
    #endif

    edge * gIt;
    for(int v=0;v<vtxNo;v++){
        gIt = g[v];
        while (gIt!=NULL){
            if(gIt->i<n){
                gIt->r = r[gIt->i];
            #ifdef NO_DP_KP_BND
              if(gIt->r > 0)
                for(int ii=0;ii<gIt->b;ii++)
                  eflMerge(gIt->w, gIt->r);
            #else
              if(gIt->r > 0&&gIt->vtx1<=gIt->vtx2)
              for(int ii=0;ii<gIt->b;ii++)
                for(int i=C;i>=0;i--){
                  if(i+gIt->w <= C)
                    mostEffRet[i+(int)gIt->w] = max( mostEffRet[i+(int)gIt->w],
                                                     mostEffRet[i]+gIt->r);
                }
            #endif
            }
            gIt = gIt->nxt;
        }
    }

    #ifndef NO_DP_KP_BND
      for(int ii=1;ii<=C;ii++)
          if(mostEffRet[ii]==0)
              mostEffRet[ii] = mostEffRet[ii-1];
    #endif

}

//Returns the maximum profit that can be achieved with a weight of wRes
double pMaxResidual(int wRes){
    #ifdef NO_DP_KP_BND
      double w = 0;
      int prf  = 0;
      effList*tmp = efl;
      while(tmp!=NULL){
          w   += tmp->w;
          if(int(w)>wRes)
              return prf + ( ((double) (wRes- (w-tmp->w)) )/ tmp->w) * efl->p;
          prf += efl->p;
          tmp = tmp->nxt;
      }
      return prf;         //all items together do not add up to wRes
    #else
        return mostEffRet[wRes];
    #endif
}

void printGraph(){
    edge* gIt;
    clog<<"Print graph:\n";
    for(int v=0;v<vtxNo;v++){
        gIt = g[v];
        while (gIt!=NULL){
            clog<<" v1="<<gIt->vtx1<<", v2="<<gIt->vtx2<<":";
            clog<<"w="<<gIt->w<<" ,";
            clog<<"b="<<gIt->b<<" ,";
            clog<<"idx="<<gIt->i<<" ,";
            clog<<"dc="<<gIt->dc<<" ,";
            clog<<"r="<<gIt->r<<" \n";
            assert(gIt->vtx1!=gIt->vtx2);
            gIt = gIt->nxt;
        }
    }
    clog<<"===\n";
}

//goes once through all edges v1-->v2 and try to improve any cost u->v2 via u--ext cost->v=v1-->v2
//returns the number of improvements
int improveAnyShortestPath(){
    int imprvs = 0;
    int v1,v2;
    double currCost;
    edge* gIt;

    for(int v=0;v<vtxNo;v++)
            costDiff[v][v]=0;
    for(int v=0;v<vtxNo;v++)
        for(int u=0;u<vtxNo;u++){
            gIt = g[v];
            while (gIt!=NULL){
                v1       = gIt->vtx1;
                assert(v==v1);
                v2       = gIt->vtx2;
                currCost = gIt->dc;
                assert(currCost>=0);
                if(costDiff[u][v2]>costDiff[u][v1]+currCost){
                    costDiff[u][v2] = costDiff[u][v1]+currCost;
                    imprvs++;
                }           
                gIt = gIt->nxt;
            }
        }    //end second for, and implicitly the first
    return imprvs;
}
//Ret variables: matrix costDiff is filled by computing a all pairs shortest path via Roy-Floyd
void buildCostDiff(){
    do{
    }while(improveAnyShortestPath());
}

void printCostDiff(){
    for(int i=0;i<vtxNo;i++){
        for(int j=0;j<vtxNo;j++){
            clog<<setw(3)<<costDiff[i][j]<<" ";
        }
        clog<<endl;
    }
}
//returns an upper bound on variable i that represents edge [v1-v2] with weight weight
double getVarBound(int i, int v1, int v2, double weight){
    assert(weight>0);
    return(costDiff[dep][v1]+costDiff[dep][v2])//Basic route: go as fast to v1, serve i, and back
;}
/*=========================================RAY SUB PROBLEM========================================*/

double cBest;                                  //Global variables to store the best solution
int    pBest;                                  //tBest = cBest/pBest
                                               //optState also declared below
int *  aBest=NULL;
typedef struct servSegment {                   //service segment for Stage 1
    int p;
    double minW;
    struct servSegment* nxt; 
    struct servSegment* prv;
    int ei;                                     //the last edge index that leads to this state
    int eu;                                     //edge from u to
    int ev;                                     //v (u and v might be redundant)
    int moreWays;                               //more paths lead here, at this minw/p

    servSegment(int p_, double w_, struct servSegment* nxt_, struct servSegment* prv_,
                int e_, int u_, int v_):
                p(p_), minW(w_), nxt(nxt_), prv(prv_), ei(e_), eu(u_), ev(v_), 
                moreWays(0) {};
}servSeg;

servSeg*** profDiff=NULL;

typedef struct prunedSeg_{                    //a structure used to record the pruned states
    servSeg * ptr;                                //that need to be recorded to reconstruct
    prunedSeg_ * nxt;                         //the route in the end
    prunedSeg_ (struct prunedSeg_* nxt_):
        nxt(nxt_){};
} prunedSeg;
prunedSeg* prnSegs=NULL;


int iter;                                       //iteration of the main do-while loop (see paper)
typedef struct state_ {                         //main algorithm state combining profit/weigh&costs
    int p;
    double minW;
    double cost;
    struct state_* nxt; 
    struct state_* prv;
    int vSt;                                    //the last edge is vSt-->vEnd; if -1=>segEnd!=NULL
    int vEnd;                                   //it is usually correct, no -1 val as for vSt
    int lastIter;                               //iter of last update; -1 => state is deleted
    servSeg* segEnd;                            //the segment that brought the vehicle here
    #ifdef LISTOFPROFITS
    int prunedAlr ;
    #endif
    void newBestState();
    state_(int p_, double w_, double c_, struct state_* nxt_, struct state_* prv_,
           int vSt_, int vEnd_, int it_, servSeg*s_):
                p(p_), minW(w_), cost (c_), nxt(nxt_), prv(prv_), 
                vSt(vSt_), vEnd (vEnd_), lastIter(it_), segEnd(s_) {
        #ifdef LISTOFPROFITS
            prunedAlr = 0;
        #endif
        if(p>0&&vEnd==dep){                      //only strictly positive profits are useful
            if(pBest==0){                        //first item ever
                pBest = p;
                cBest = cost;
                newBestState();
            } // cost/p <cBest/pBest
            else if (cost*pBest<cBest*p||(cost==0&&cBest==0&&p>pBest)){
                pBest = p;
                cBest = cost;
                newBestState();
            } 

            //Commented below to avoid slowdowns
            //clog<<"new sstate at iter="<<iter<<"from "<<vEnd<<"with p="<<p<<",minW="<<minW
            //<<", cost="<<cost;
            //#ifndef NO_DP_KP_BND
            //    clog<<", mostEffRet="<<mostEffRet[(int)(C-minW)];
            //#endif
            //clog<<" pBest="<<pBest<<" cBest="<<cBest<<"time="<<timeCpuPassedMSec()<<endl;
        }
    };
}state;
state** statesDP=NULL;                          //one-dimensional table
state*  optState;                               //null only if variable limits are generating t max

#ifdef LISTOFPROFITS
typedef struct profit_{
    int p;
    state* prvState;
    struct profit_* nxt;
    profit_(int p_, state* prvState_, struct profit_ * nxt_):    
        p(p_), prvState(prvState_), nxt(nxt_){ };
}profit;

profit** prfList;                                //A list of profits
#endif

void state::newBestState(){
    optState = this;
}
typedef struct prunedState_{                    //a structure used to record the pruned states
    state * ptr;                                //that need to be recorded to reconstruct
    prunedState_ * nxt;                         //the route in the end
    prunedState_ (struct prunedState_* nxt_):
        nxt(nxt_){};
} prunedState;
prunedState* prnStates=NULL;

//delete any previous states in profDiff to realease memory
void delProfDiff(){
    servSeg * it, *itNext ;
    if(profDiff==NULL)
        return;
    for(int v=0;v<vtxNo;v++){
        for(int u=0;u<vtxNo;u++){
            it = profDiff[v][u];
            while(it!=NULL){
                itNext = it->nxt;
                delete it;
                it=itNext;
            }
        }
        delete[] profDiff[v];
    }
    delete[] profDiff; 
    #ifdef NO_DP_KP_BND
      effList*tmp;
      while(efl!=NULL){
          tmp = efl->nxt;
          delete efl;
          efl=tmp;
      }
    #else
      delete[] mostEffRet;
    #endif
    if(prnSegs==NULL)
        return ;
    prunedSeg * prnIt = prnSegs;
    prunedSeg * next;
    while(prnIt!=NULL){
        delete prnIt->ptr;
        next = prnIt->nxt; 
        delete prnIt;
        prnIt = next;
    }
}

#ifdef LISTOFPROFITS
void delPrfList(){
    profit* tmpNext;
    for(int v=0;v<vtxNo;v++){
        while(prfList[v]!=NULL){
            tmpNext = prfList[v]->nxt;
            delete prfList[v];
            prfList[v] = tmpNext;
        }
    }
    delete prfList;
}
#endif

//delete any previous states in statesDP (main alg states) to release memory
void delStates(){
    state * it,*itNext;
    if(statesDP==NULL)
        return;
    for(int v=0;v<vtxNo;v++){
        it = statesDP[v];
        while(it!=NULL){
            itNext = it->nxt;
            #ifdef LISTOFPROFITS
            if(!it->prunedAlr)
            #endif
            delete it;
            it=itNext;
        }
    }
    delete[] statesDP; 
    if(prnStates==NULL)
        return ;
    prunedState * prnIt = prnStates;
    prunedState * next;
    while(prnIt!=NULL){
        delete prnIt->ptr; 
        next = prnIt->nxt; 
        delete prnIt;
        prnIt = next;
    }

    
}

//prints profDiff info
void printProfDiff(){
    servSeg * it;
    if(profDiff==NULL)
        return;
    for(int u=0;u<vtxNo;u++){
        for(int v=0;v<vtxNo;v++){
            it = profDiff[u][v];
            if(it!=NULL)
                clog<<u<<","<<v<<"->";
            while(it!=NULL){
                clog<<setw(4)<<"p="<<setw(5)<<it->p<<": w="<<setw(5)<<it->minW;
                if(it->prv==NULL){
                    assert(u==v);
                    clog<<": dummy edge";
                }else{
                    clog<<": via edge="<<it->ei<<"["<<it->eu<<","<<it->ev<<"]";
                    servSeg* it2 = it->prv;
                    if(it2!=NULL)
                        clog<<" from a prv state with p,w="<<it2->p<<","<<it2->minW
                             <<" and edge [eu,ev] with eu,ev"<<it2->eu<<","<<it2->ev;
                    else
                        clog<<" from a null state on "<<it->eu;
                    assert(it->ev==v);
                }
                clog<<endl;
                it = it->nxt;
                if(it!=NULL)
                    clog<<"     ";
            }
        }
    }
    clog<<"===========================================\n";
}


//prints states info
void printStates(){
    state * it;
    if(statesDP==NULL)
        return;
    for(int u=0;u<1;u++){
        it = statesDP[u];
        clog<<setw(3)<<u;
        while(it!=NULL){
            if(it!=statesDP[u])
                clog<<"   ";
            clog<<setw(4)<<"p="<<setw(5)<<it->p<<": w="<<setw(5)<<it->minW
                 <<"cost="<<setw(4)<<it->cost<<"iter="<<setw(4)<<it->lastIter;
            if(it->prv==NULL){
                clog<<": dummy no-move state";
            }else{
                clog<<": from a state ending in="<<it->prv->vEnd;
            }
            it = it->nxt;
            if(it!=NULL)
                clog<<endl;
        }
        clog<<endl;
    }
    clog<<"===========================================\n";
}


//input: a service segment starting in u, ending in v, edge (index e) from v to v2
//return value: 1 success, 0 it is not efficient to insert it
int augmentServ(int u, int v, servSeg* s, edge* e){
    assert(v==e->vtx1);
    assert(v==s->ev||s->ev<0);                    //s ends in s->ev=v or it is a NULL service
    int v2 = e->vtx2;
    if(s->ei==e->i)                               //forbit 2-cycles: this is safe because if there
        return 0;                                 //are two equivalent ways A and B to link u and v,
                                                  //route u--A-->v--B-->u--A-->B... *is* (always) generated
    int    pNew = s->p    + e->r;
    double wNew = s->minW + e->w;

    if(wNew>C)
        return 0;

    if(profDiff[u][v2]==NULL){
        profDiff[u][v2] = new servSeg(pNew,wNew,NULL,s,e->i,v,v2);
        return 1;
    }
    servSeg* it = profDiff[u][v2];
    servSeg* itPrv = it;

    while((it!=NULL)
         &&((it->p<pNew)||(it->prv==NULL))){      //jump over smaller profits or the dummy route
        if(it->minW>=wNew){                       //'it' is a lower-profit segment with higher weight
            if(it==itPrv){                        //erase 'it' -> the head list case
                profDiff[u][v2] = it->nxt;
                it=it->nxt;
                prnSegs = new prunedSeg(prnSegs); //same objective as 
                prnSegs->ptr = itPrv;             //delete itPrv;
                itPrv = it;
            }else{                                //erase 'it' -> the non-head list case
                itPrv->nxt = it->nxt;
                prnSegs = new prunedSeg(prnSegs); //same objective as
                prnSegs->ptr = it;                //delete it
                it = itPrv->nxt;
            }
            
        }
        else{
            itPrv = it;
            it = it->nxt;
        }
    }
    if( (it!=NULL) && (it->p==pNew) ){            //reached again the same profit
        //I think this stays more correctly here than below, OR PUT IT BELOW?
       // if(wNew<=it->minW*1)
        it->moreWays = 1;
        if(wNew>=it->minW)
            return 0;

        it->minW = wNew;                          //replace state: the new one has smaller weight
        it->prv  = s;
        assert(s->minW<wNew);
        it->ei   = e->i;
        it->eu   = v;
        assert(it->ev==v2);
        return 1;
    }
    //'it' is either NULL or higher-profit
    if(it!=NULL&&wNew>=it->minW)                  //new seg: load(wNew) > 'it', profit < it
        return 0;

    if(itPrv==profDiff[u][v2]&&it==itPrv){        //change head if necessary
        profDiff[u][v2] =  new servSeg(pNew,wNew,it,s,e->i,v,v2);
    }
    else{
        servSeg * sCopy = new servSeg(pNew,wNew,it,s,e->i,v,v2);
        itPrv -> nxt = sCopy;
    }
    return 1;
}

//return value: the number of updates in this round
int oneSrvRound(){
    int updates = 0;

    edge *gIt;                                             //graph iterator
    servSeg * it;                                          //service iterator
    for(int v=0;v<vtxNo;v++){                              //for all edge v->(v2)
        gIt = g[v];
        while(gIt!=NULL){
            for(int u=0;u<vtxNo;u++){                      //for each segment u->v
                it = profDiff[u][v];
                while(it!=NULL){
                    updates += augmentServ(u,v,it,gIt);
                    it = it->nxt;
                }
            }
            gIt = gIt->nxt;
        }
    }

    return updates;
}

//Ret variables: profDiff is filled by computing all segments u->v (all u,v \in V).
void buildProfDiff(){
    int u,v;
    //1. allocate new memory and add (0,0) to positions [v,v];
    prnSegs = NULL;
    profDiff = new servSeg**[vtxNo];
    for(u=0;u<vtxNo;u++){
        profDiff[u] = new servSeg*[vtxNo];
        for(v=0;v<vtxNo;v++)
            profDiff[u][v] = NULL;
        profDiff[u][u] = new servSeg(0,0,NULL,NULL,-1,-1,-1);//null service, edge indexes negatives
    }
    //go through all states and try to continue service
    do{
        //find best segment and prune states by solving knapsack
        //printProfDiff();
    }while(oneSrvRound()>0);
    
}

//Adds in a all the edge indexes that let to endpoint segment (profit/Weight)
void reconstructSeg(servSeg * endPoint, int*a){
    int deadNext=0;
    while(endPoint->prv!=NULL){                             //->prv=NULL -> got to dummy edge
        assert(!deadNext);
        if(!(endPoint->prv->minW<endPoint->minW))
            deadNext = 1;
        assert(endPoint->ei<n);
        a[endPoint->ei]++;
        endPoint = endPoint->prv;
    }
}
//returns 1 if state it is dominated by *a higher profit state* of cost cost and weight minW
int dominatedBy(state * it, double cost, double minW, int p){
     if(it->p > p)  
        return 0;
     if((it->cost>=cost)                              //'it' is dominated by newstate with
             &&(it->minW >= minW) )                   //higher profit, lower cost, lower weight
        return 1;
    return 0;       
}
//Returns 1 if there is no use (no chance to lower t) to have a state with profit p and weight w
int prune(int p, double w, double c){
    //Prunning condition cBest/pBest < c/(p+(C-w)*mostEffRet) OR:
    if ( cBest * (p + pMaxResidual((int)(C-w)) ) < c * pBest)
        return 1;
    return 0;
}
//Returns 1 if the state is to be pruned
int prune(state*it){
    if(it->lastIter<iter-1)
        return 1;
    if(prune(it->p, it->minW, it->cost))
        return 1;
    //if((it->nxt!=NULL)&&dominatedBy(it,it->nxt->cost,it->nxt->minW,it->nxt->p))
    // return 1;                                           //CUT IT, it does not seem very useful
    return 0;
}

state*prv;

//returns the state that is added/updated (ie, if it verifies all conditions), or NULL when no state
//is updated.
state* newState(int p, double minW, double cost, int vEnd){
    if(minW>C)
       return NULL; 
    if(prune(p,minW,cost))
        return NULL;
    if(minW>C){
        return NULL;
    }
    state * it    = statesDP[vEnd];
    state * itPrv = statesDP[vEnd];
    #ifdef LISTOFPROFITS
  
      profit* prfIt = NULL;
      profit* prfItPrv= NULL;
  
      if((it!=NULL)&&(it->p<p)){                             //search directly to profit value if it=
          prfItPrv= prfList[vEnd];                           //=statesDP[vEnd] is not the searched val
          prfIt = prfList[vEnd];
          while(prfIt!=NULL&&prfIt->p<p){
              if(prfIt->prvState!=NULL){
                  if(prfIt->prvState->prunedAlr){            //do not allow a pruned state as
                        //while((prfIt->prvState!=NULL)        //previous state to start exploring
                        //     &&(prfIt->prvState->prunedAlr)
                        //     &&(prfIt->prvState->p<p))
                        //        prfIt->prvState = prfIt->prvState->nxt;
                        //if(prfIt->prvState==NULL||prfIt->prvState->p>=p)
                            prfIt->prvState = itPrv;
                            if(itPrv!=NULL)
                            if(!(itPrv->p<prfIt->p))
                                prfIt->prvState = NULL;
                            //?? check better if below should be added
                            //if(itPrv==statesDP[vEnd])
                            //  it = itPrv;
                  }
                  if((prfIt->prvState!=NULL)
                    &&(!prfIt->prvState->prunedAlr)
                    &&(prfIt->prvState->nxt!=NULL)
                    &&(!prfIt->prvState->nxt->prunedAlr)
                    &&(prfIt->prvState->nxt->p<=p)
                    &&dominatedBy(prfIt->prvState->nxt,cost,minW,p)
                    ){
                        state*it2Prv = prfIt->prvState;
                        state*it2    = it2Prv->nxt;
                        assert(!it2->prunedAlr);
                        prnStates      = new prunedState(prnStates);
                        prnStates->ptr = it2;
                        it2->prunedAlr = 2;
                        assert(it2Prv!=it2);
                        it2Prv->nxt = it2->nxt;
                  }
              }
              itPrv = prfIt->prvState;
              if(itPrv!=NULL)
                if(!(itPrv->p<=p))
                    {
                    itPrv = NULL;
                    prfIt -> prvState = NULL;
                }
              //      cout<<cc<<endl;
              prfItPrv= prfIt;
              prfIt = prfIt->nxt;
          }
          if(itPrv!=NULL){                                   //itPrv is found and correct
              it=itPrv->nxt;                                 
          }
          else{
              it    = statesDP[vEnd];
              itPrv = statesDP[vEnd];
          }
  
           if(prfIt!=NULL){
            while(it!=NULL&&(it->p<p||it->prv==NULL)){       //update itPrv if necessary 
               itPrv = it;
               it    = it->nxt;
            }
               prfIt->prvState = itPrv;
               //erase some profit values: you can disable below
               //if((itPrv!=NULL)
               //   &&(itPrv->nxt!=NULL)
               //   &&(itPrv->nxt->p > prfIt->p)){
               //   assert(prfItPrv!=NULL);
               //   prfItPrv->nxt =  prfIt->nxt;
               //}
           }
      }
    #endif
    if( (itPrv!=NULL)&&(it!=NULL) ) 
        if(!((it==itPrv&&it==statesDP[vEnd])||(itPrv->p<p)||(itPrv->prv==NULL))){
               itPrv = it;
               it    = it->nxt;
        }
    while((it!=NULL)                                       //jump over
          &&((it->p<p)                                     //smaller profit
            ||(it->p==p&&it->cost<cost)                    //same profit, smaller cost
            ||(it->prv==NULL)) ){                          //dummy state
        //Condition to prune the state it
         
        if(it->cost==cost&&it->minW > minW){               //CUT ">=" (put >) as it is not correct.
                #ifdef LISTOFPROFITS
                if(!it->prunedAlr){
                    it->prunedAlr = 1;
                #endif
                prnStates      = new prunedState(prnStates);
                prnStates->ptr = it;
                #ifdef LISTOFPROFITS
                }
                #endif
                if(itPrv==it&&it==statesDP[vEnd]){         //Change head
                    it = it->nxt;
                    #ifdef LISTOFPROFITS
                    while( (it!=NULL)&&(it->prunedAlr)) it = it->nxt;
                    #endif
                    statesDP[vEnd] = it;
                    itPrv = statesDP[vEnd];
                    it = itPrv;
                    continue;
                }else{
                    it = it->nxt;
                    #ifdef LISTOFPROFITS
                    while( (it!=NULL)&&(it->prunedAlr)) it = it->nxt;
                    #endif
                    itPrv->nxt = it;
                    it = itPrv;
                }
        }

        if((it->prv!=NULL)&&(it->p==p)&&(it->minW<=minW))  //existing state has same profit, lower
            return NULL;                                   //cost (implicitly), lower profit->better
        itPrv = it;
        if(!((itPrv->p<=p)||(itPrv->prv==NULL)))
            cerr<<"Warning: profit of itPrv seems strange"<<endl;
        it = it->nxt; 
        #ifdef LISTOFPROFITS
            while(it!=NULL&&(it->prunedAlr))
                it = it->nxt; 
        #endif
        itPrv->nxt = it;
        #ifdef LISTOFPROFITS
            if(it!=NULL&&prfIt!=NULL&&it->p>prfIt->p&&(itPrv->p == prfIt->p) )
                if(prfIt->nxt!=NULL)
                    prfIt->nxt->prvState = itPrv;
        #endif
    }
    if(it!=NULL)
        if(it->p==p&&it->cost==cost){
            #ifdef LISTOFPROFITS
            if(it->minW>minW&&(!it->prunedAlr)){           //get the same state, but smaller load
            #else
            if(it->minW>minW                  ){           //get the same state, but smaller load
            #endif
                it->minW = minW;
                it->lastIter = iter;
                return it;
            }
            return NULL;
        }
    state* addedState;

    if(itPrv==NULL)
        it = itPrv;
    if((itPrv==statesDP[vEnd])
        &&itPrv==it                                         //Enforcing condition 
        ){                                                  //change head if necessary
        //if(itPrv!=NULL)
        //if(!(itPrv->p>p||(itPrv->p==p&&itPrv->cost>cost)))
        //    assert(4==9);
        statesDP[vEnd] = new state(p, minW, cost, itPrv,NULL,-1, vEnd,iter, NULL);
        addedState = statesDP[vEnd];
        //state*tmp = itPrv;
        //while(tmp!=NULL){
        //    assert(p<=tmp->p);
        //    tmp = tmp->nxt;
        //}
    }else{
        if(it!=NULL)
        if(!(it->p>p||(it->p==p&&it->cost>cost)))
            assert(1==0);
        addedState   = new state(p, minW, cost, it,NULL,-1, vEnd,iter, NULL);
        if(!(itPrv->p<=p))
            assert(5==4);
        itPrv->nxt = addedState;
    }

    //the profit of addedState need to be merged into prfList. Several rather complex cases can arise
    #ifdef LISTOFPROFITS
        if(prfItPrv==NULL&&prfIt==NULL){
            if(prfList[vEnd]==NULL||prfList[vEnd]->p>p){
                prfList[vEnd] = new profit(p,NULL,prfList[vEnd]);
            }
            else{
                prfIt = prfList[vEnd];
                prfItPrv = prfList[vEnd];
                while(prfIt!=NULL&&prfIt->p<p){
                    prfItPrv = prfIt;
                    prfIt = prfIt->nxt;
                }
                if((prfIt==NULL)||(prfIt->p!=p)){
                    if(it!=itPrv&&itPrv->prv!=NULL)              //itPrv->p<p
                        prfItPrv -> nxt = new profit(p,itPrv,prfIt);
                    else
                        prfItPrv -> nxt = new profit(p,NULL,prfIt);
                }
            }
            return addedState;
        }
        if(prfIt==NULL){
            assert(prfItPrv->p<p);
            assert(prfItPrv->nxt==NULL);
            prfItPrv -> nxt = new profit(p, itPrv, NULL);
            return addedState;
        }
        if(prfIt->p>p){                                      //addedState=first state with prf p
                prfIt -> p = p;
                //keep the same: prfIt -> prvState = 
                prfIt -> nxt = new profit(prfIt->p, addedState, prfIt->nxt);
                return addedState;
        }
        prfItPrv = prfIt;
        prfIt    = prfIt->nxt;
        if(prfIt==NULL)
            return addedState;

        assert(p < prfIt->p);
        if( (prfIt->prvState==NULL)                          //next profit list has higher profit
            || (p > prfIt->prvState->p)
            ||((p==prfIt->prvState->p) && (cost > prfIt->prvState->cost)) ){
            prfIt->prvState = addedState;
        }
    #endif
    return addedState;
            
}

//starting from state src, try to reach new states by adding up service segments
//Return value: the number of states reached
int stateAddService(state*src){
    int updates = 0;
    int p;
    double minW;
    double cost;
    servSeg * segIt;

    for(int u=0;u<vtxNo;u++){
        segIt = profDiff[src->vEnd][u];
        while(segIt!=NULL){ 
                if((segIt!=NULL)&&(!segIt->moreWays)&&(src->prv!=NULL)){
                   if(src->prv->segEnd==segIt){
                      segIt = segIt->nxt;
                     continue;
                    }
                   if((src->prv->prv!=NULL)&&(src->prv->prv->prv!=NULL))
                     if(src->prv->prv->prv->segEnd==segIt){
                        segIt = segIt->nxt;
                       continue;
                     }
                }
            p    = src->p + segIt->p;
            minW = src->minW + segIt->minW;
            cost = src->cost;
            state * sNew = newState(p,minW,cost,u);
            if(sNew!=NULL){
                updates++;
                prv=src;
                sNew->prv = src;
                sNew->vSt = -1;                 //there is no "last edge", it is a serv seg who
                sNew->segEnd = segIt;           //brought us here
            }
            segIt = segIt->nxt;

            //Timeout might need to be caught here for some large instances
            #ifdef MILISECS_MAX
               if(timeCpuPassedMSec()>MILISECS_MAX){
                  cerr<<"Exit loop inside subproblem after time out\n";
                  pBest = INT_MAX;
                  return 0;
               }
            #endif
        }
    }
    return updates;
}
int stateAddCost(state*src){
    int updates = 0;
    int p;
    double minW;
    double cost;
    for(int u=0;u<vtxNo;u++){
        if(u==src->vEnd)                         //why would one do a dead-heading circle 
            continue;                            //starting and ending at src->vEnd?
        p     = src->p;
        minW  = src->minW;
        cost  = src->cost+costDiff[src->vEnd][u];
        //cout<<"call new state on"<<p<<minW<<cost<<u<<endl;
        state * sNew = newState(p,minW,cost,u);
        if(sNew!=NULL){
            updates++;
            sNew->prv = src;
            sNew->vSt = src->vEnd;               //last edge is src->vEnd....->u
            sNew->segEnd = NULL;
        }

       //Timeout might need to be caught here for some large instances
       #ifdef MILISECS_MAX
          if(timeCpuPassedMSec()>MILISECS_MAX){
             cerr<<"Exit loop inside subproblem after time out\n";
             //cBest = 1;
             pBest = INT_MAX;
             //aBest = NULL;
             return 0;
          }
       #endif
    }
    
    return updates;
}
int oneIteration(){
    #ifdef LISTOFPROFITS
    prfList   = new profit*[vtxNo];        //No list of profits
    for(int v=0;v<vtxNo;v++)
        prfList[v]  = NULL;
    #endif

    int updates = 0;
    state * it;
    state * itPrv;
    for(int v=0;v<vtxNo;v++)
        if(statesDP[v]!=NULL){
            it    = statesDP[v];
            itPrv = statesDP[v];
            while(it!=NULL){                               //useless states are destroyed
                if(prune(it)){
                    #ifdef LISTOFPROFITS
                    if(!it->prunedAlr){
                        it->prunedAlr = 1;
                    #endif
                    prnStates      = new prunedState(prnStates);
                    prnStates->ptr = it;
                    #ifdef LISTOFPROFITS
                    }
                    #endif

                    if(itPrv==statesDP[v]){                //head replacement
                        statesDP[v] = it->nxt;
                        itPrv       = statesDP[v];
                        it          = itPrv;
                    }else{                                 //middle elem replacement
                        itPrv->nxt     = it->nxt;
                        it             = it->nxt;
                    }
                }else{
                    if(it->lastIter==iter-1){
                        assert(v==it->vEnd);
                        if(iter%2==1)
                            updates+=stateAddService(it);
                        else
                            updates+=stateAddCost(it);
                    }
                    itPrv = it;
                    it = it->nxt;
                }
            }
        }
    
    #ifdef LISTOFPROFITS
      delPrfList();
    #endif
    return updates;
}

void buildStates(){
    iter = 0;
    statesDP  = new state*[vtxNo];
    pBest = 0;                             //no profit found yet
    cBest = 0;
    optState  = NULL;                      //no optimal state found
    prnStates = NULL;
    for(int v=0;v<vtxNo;v++){
        statesDP[v] = NULL;
    }
    statesDP[dep] = new state(0,0,0,NULL,NULL,-1,dep,iter,NULL);

    iter++;
    oneIteration();
    statesDP[dep]->lastIter ++;            //the dummy state stays there to allow direct deadheading

    while( (!((pBest>0)&&(cBest==0)))      //stop if cBest=0 for a real route (pBest>0)
            //&& (iter/2<n))               //or more iterations than edges to serve
    ){             
        #ifdef MILISECS_MAX
           if(timeCpuPassedMSec()>MILISECS_MAX){
              cerr<<"Exit subproblem"<<currInst<<" after time out\n";
              //cBest = 1;
              pBest = INT_MAX;
              break;
           }
        #endif
        iter++;
        if(!  oneIteration())              //no more states available, break
            break;

    };
    clog<<"cBest/pBest="<<cBest<<"/"<<pBest<<endl;
}


//Reconstruct the optimal solution in a
void reconstrOptConfig(int* &a, int*r){
    if(optState==NULL){                    //no a-route yielding better ca/p that basic var bounds
        clog<<"Null optstate with cBest/pBest="<<cBest<<"/"<<pBest<<endl;
        return;
    }
    if(a==NULL){
        a = new int[n];
        for(int i=0;i<n;i++) a[i] = 0;
    }
    while(optState->prv!=NULL){           //the dummy state only has prv=NULL
        if(optState->segEnd!=NULL)
           reconstructSeg(optState->segEnd, a); 
        else
           clog<<"Deadheading:" <<optState->vSt<<"->"<<optState->vEnd
               <<" c="<<costDiff[optState->vEnd][optState->vSt]<<endl;
        optState = optState->prv;
    }
    for(int i=0;i<n;i++)
        if(a[i]!=0)
            clog<<i<<"edg:"<<a[i]<<" tms,r[i]="<<r[i]<<"; ";
    clog<<endl;
}

//Find max t (max pBest with cBest=1) s.t. rt doesn't go beyond the lower limit lb[i] of any var i
void tMaxFrmVarLimts(int * r){
    for(int i=0;i<n;i++)
        if(r[i]<0){
            cBest = 1;                           //when searching natural bounds, costs are 1
            if(lb[i]==0)                          //one can not advance at all on direction r
                pBest = INT_MAX;
            if(pBest < r[i]/lb[i])
                pBest = r[i]/lb[i];
            optState = NULL;
        }
}

void irmSubproblem(int * r, double &t, int * &a, double &ca, int& p){
    //How to test, generate this automatically: r[0]=-1;r[1]=3;r[2]=0;r[3]=6;r[4]=0;r[5]=7;r[6]=4;r[7]=2;r[8]=1;r[9]=6;r[10]=-1;r[11]=20;r[12]=-1;r[13]=10;r[14]=3;r[15]=4;r[16]=8;r[17]=9;r[18]=7;r[19]=9;r[20]=26;r[21]=2;r[22]=16;r[23]=1;r[24]=1;r[25]=2;r[26]=1;r[27]=2;r[28]=1;r[29]=0;r[30]=0;r[31]=0;r[32]=0;r[33]=2;r[34]=15;r[35]=1;r[36]=13;r[37]=-1;r[38]=16;r[39]=7;r[40]=7;r[41]=2;r[42]=0;r[43]=20;r[44]=2;r[45]=0;r[46]=7;r[47]=2;r[48]=1;r[49]=0;r[50]=2;
    //for(int i=0;i<n;i++)
    //    clog<<"r["<<i<<"]="<<r[i]<<";";
    clog<<"ARP IRM SUBProb\n";
    tMaxFrmVarLimts(r);
    rCpyToGraph(r);
  
    buildProfDiff();
    //printProfDiff();


    buildStates();
    //printStates();
   
    aBest = NULL;
    reconstrOptConfig(aBest,r); 

    delStates();
    delProfDiff();

    
    a=aBest;
    p=pBest;
    ca=cBest;
    if(p==0)
        t=0;
    else 
        t=ca/p;

    assert(aBest!=NULL);
}




}    //end namespace problemSpecific
