#include <cstdlib>
#include <vector>
#include <cmath>
using namespace std;

#include <ilcplex/ilocplex.h>                    //Cplex header

#include "colGenerator.h"

struct cplexCheshireData{
    IloEnv         env;
    IloModel       model;
    IloNumVarArray vars;
    IloCplex       cplex ;
    cplexCheshireData(int n):
            env(), model(env), vars(env,n,-IloInfinity,IloInfinity), cplex (env) {};

};
colGenerator::colGenerator(int nrVars): 
                           n(nrVars), upBound(INT_MAX), d(*new cplexCheshireData(n)){


    d.cplex.extract(d.model);
    #ifdef NO_CPLEX_OUTPUT
    d.cplex.setOut(d.env.getNullStream());
    #endif 
    duals = new double[n];
};
colGenerator::~colGenerator(){ 
    d.vars.end();
    d.model.end();
    d.env.end();
    delete[] duals;
}
void colGenerator::setVarBounds(double * varLb, double * varUb){
    IloNumArray lb (d.env,n);
    IloNumArray ub (d.env,n);
    for (int i=0;i<n;i++){
        d.model.add(d.vars[i]>=varLb[i]);
        lb[i] = varLb[i];
        ub[i] = varUb[i];
    }
    d.vars.setBounds(lb,ub);
}
void colGenerator::setVarBounds(double varLb, double  varUb){
    IloNumArray lb (d.env,n);
    IloNumArray ub (d.env,n);
    for (int i=0;i<n;i++){
        lb[i] = varLb;
        ub[i] = varUb;
    }
    d.vars.setBounds(lb,ub);
}
void colGenerator::setObjective(double * coefs){
    int i;
    IloExpr expr(d.env);
    for(i=0;i<n;i++)
        if(abs(coefs[i])>EPSILON)
            expr+=coefs[i]* d.vars[i];
    IloObjective obj(d.env, expr, IloObjective::Maximize);
    d.model.add(obj);
    expr.end();
}

void colGenerator::modelAddColumn(double * coefs, double rightHand){
    int i;
    IloExpr expr(d.env);
    for(i=0;i<n;i++)
        if(abs(coefs[i])>EPSILON)
            expr+=coefs[i]* d.vars[i];
    d.model.add(expr<=rightHand);
    expr.end();    
}


void colGenerator::modelAddColumn(int * coefs, double rightHand){
    int i;
    IloExpr expr(d.env);
    for(i=0;i<n;i++)
        if(abs(coefs[i])>EPSILON)
            expr+=coefs[i]* d.vars[i];
    d.model.add(expr<=rightHand);
    expr.end();    
}


double colGenerator::solve(){
    d.cplex.solve();
    double objVal = d.cplex.getObjValue();   //Extract solution
    
    if(d.cplex.getStatus()==IloAlgorithm::Infeasible){
        cerr<<"Attention: finding infeasible solution. Returning -INT_MAX";
        objVal = -INT_MAX;
    }

    IloNumArray solution(d.env);
    d.cplex.getValues (solution,d.vars);
    for(int i=0;i<n;i++)
            duals[i] = solution[i];
    
    if(objVal<upBound)                     //Update bound
        upBound = objVal;
    return upBound;
}
void colGenerator::getDuals(double *ystar){
    int i;
    if(ystar!=NULL)
    for(i=0;i<n;i++)
            ystar[i] = duals[i];
}

double colGenerator::coefRedCostContrib(int coef, int idx){
//    if(abs(coef)<EPSILON) //Useless when coef is integer
//        return 0;
   return (double)coef * duals[idx]; 
}

double colGenerator::redCostSum(int * coefs){
    double totalSum = 0;
    for (int i=0;i<n;i++)
        totalSum += coefRedCostContrib(coefs[i],i);
    return totalSum;
}

int colGenerator::negRedCostColumn(int * coefs, double rightHand){
    double redCost = redCostSum(coefs);
    return (redCost > rightHand + EPSILON);
}
void colGenerator::exportModel(char* filename){
    d.cplex.exportModel(filename);
}

/*                                   NOT DIRECTLY RELATED                                  */
//Solve a classical simplex table with the following notations/conventions: n vars, m constraints
//only non-negative integers in the table and all constraints are: <=rightHandSideAll 
//returns the objective value, puts solution in ystar (if not NULL)

double solveStdSimplexTable(int n, int*objFunc, double rightHandSideAll, 
                           int m,  vector<int*>  table, double*ystar){
    //Attention to calling size(): the documentation does not guarantee a O(1) time!!
    //assert(m==(int)table.size()); 
    int i,j;
    IloEnv env;
    IloModel model(env);

    IloNumVarArray x(env, n, 0, rightHandSideAll);
    
    IloExpr expr(env);
    for(i=0;i<n;i++)
        expr+=objFunc[i]* x[i];
    IloObjective obj(env, expr, IloObjective::Maximize);
    model.add(obj);
    IloConstraintArray constr(env);
    for(i=0;i<m;i++){
        IloExpr e(env);
        for(j=0;j<n;j++){
            e+=table[i][j]*x[j];
        }
        constr.add(e<=1);
        e.end();    
        model.add(constr[i]);
    }
    //Solve model
    IloCplex cplex (env);
    cplex.extract(model);
    cplex.setOut(env.getNullStream());
    cplex.solve();

    //Extract solution
    double objVal = cplex.getObjValue();
    IloNumArray solution(env);
    cplex.getValues (solution,x);
    if(ystar!=NULL)
        for(i=0;i<n;i++){
            ystar[i] = solution[i];
        }

    //Clean all and return
    constr.end();
    x.end();
    model.end();
    env.end();
    return objVal; 
}
