#ifndef CPLEX_H_INCLUDED
#define CPLEX_H_INCLUDED



//this file should be stand-alone, so we define a specific epsilon
#define EPSILON 1.0e-6             

//Comment or uncomment below to disable terminal output
#define NO_CPLEX_OUTPUT
class colGenerator{
    private:
    int n;
    double * duals;
    double upBound;
    class cplexCheshireData & d;    //d-reference (see d-pointer, compiler firewall or cheshire data)
                                    //need no cplex include outside cplex.cpp, faster compilation
                                    //At the file bottom,you find the stack-overflow post used to it
    public:
    //constructs object using nrVars variables, each one bounded above by varUB(INT_MAX)
    colGenerator(int nrVars);
    ~colGenerator();
    //the lower and upper bound on each variable is defined by *arrays* varLB and resp varUb
    void setVarBounds(double*varLB, double*varUB);
    //the lower and upper bound on each variable is defined by *doubles* varLB and resp varUb
    void setVarBounds(double varLB, double varUB);
    //Set the objective function
    void setObjective(double * coefs);
    //Add to the model a new column (in the dual solving approach, it is stored as a row)
    void modelAddColumn(double * coefs, double rightHand);
    //Add to the model with integer column coefficients
    void modelAddColumn(int * coefs, double rightHand);

    //the contribution in the reduced cost of coef at position idx
    double coefRedCostContrib(int coef, int idx);
    //given newColCoefs (coefficients of a new column), calc. \sum newColCoefs[i]duals[i]
    double redCostSum(int * newColCoefs);
    //given newColCoefs (coefficients of a new column) and right hand, check dual feasibility
    int    negRedCostColumn(int * newColCoefs, double rightHand);
    //returns the upper bound associated to the new optimized solution 
    double solve();
    //puts the dual variables in *yStar (if non null)
    void getDuals(double*ystar);
    //saves the model in a file
    void exportModel(char* filename);
};


///USAGE EXAMPLE
///#include <iostream>
///#include "colGenerator.h"                               //colGenerator
///using namespace std;
///
///int main(){
///    double b[2];b[0]=3, b[1]=3; 
///    colGenerator cg(2);   
///    double l[2];l[0]=1, l[1]=1; 
///    double u[2];u[0]=2, u[1]=5; 
///    cg.setVarBounds(l,u); 
///    cg.setObjective(b);
///    double max = cg.solve(); 
///    cout<<max;    
///    double*y = new double[2]; 
///    cg.getDuals(y); cout<<"  sol:"<<y[0]<<"n"<<y[1]<<endl;
///    
///    double*pattern = new double[2]; 
///    pattern[0]=1; pattern[1]=2;
///    cg.modelAddColumn(pattern,3);
///    pattern[1]=1; pattern[0]=2;
///    cg.modelAddColumn(pattern,3);
///
///    cout<<cg.solve();
///    cg.getDuals(y); cout<<"  sol:"<<y[0]<<","<<y[1]<<endl;
///    cg.exportModel("tmp.lp");
///}

/*                                   NOT DIRECTLY RELATED                                  */
/*                                  (uncomment to use it)                                  */
//Solve a classical simplex table with the following notations/conventions n vars, m constraints
//only non-negative integers in the table and all constraints are: <=rightHandSideAll 
//returns the objective value, puts solution in ystar (if not NULL)
//double solveStdSimplexTable(int n, int*objFunc, double rightHandSideAll, 
//                           int m,  vector<int*>  table, double*ystar);
#endif
	

/// d-pointers are one implementation, among many, of the pimpl pattern. It is also one of the early
/// implementations: "The name 'd-pointer' stems from Trolltech's Arnt Gulbrandsen, who first introduced
/// the technique into Qt, making it one of the first C++ GUI libraries to maintain binary compatibility
/// even between bigger release." Source
/// 
/// One advantage of using macros is the option of changing some implementation details of the pattern
/// implementation in a central place at compile time. You could for example design your macros to leave
/// you the option of switching to the fast pimpl implementation at a later time without changing tons
/// of code (hopefully you won't need this if you are using pimpl :-)). Provided that you made no
/// mistakes in your macro design/implementation...
/// 
/// However, I would personally recommend avoiding macros for your pimpl implementation as they are
/// cryptic for any newcomer to your source tree. Macros create magical dialects that are often
/// error-prone and not as meaningful as the original source code. They also come with all the problems
/// associated with the C Pre Processor; it's unaware of the underlying language.
/// 
/// Personally I like to use what I call a d-reference. Instead of a pointer, you use a reference and
/// you don't have to d-reference. 8-) It looks something like this:
/// 
/// // MyClass.h
/// 
/// class MyClass
/// {
/// public:
///     MyClass();
///     ~MyClass();
/// 
///     // implementation methods
/// 
/// private:
///     class MyClassPrivate& d;
/// };
/// 
/// // MyClass.cpp
/// 
/// struct MyClassPrivate
/// {
///     int x;
/// };
/// 
/// MyClass::MyClass()
/// : d(*new MyClassPrivate)
/// {
/// 
/// }
/// 
/// MyClass::~MyClass()
/// {
///     delete &d;
/// }
/// 
/// // In methods use d.x
/// 
/// 

