#include "config.h"
#include <string.h>
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cassert>
#include <cstdlib>
#include <climits>
#include <cmath>
#include "subprob.h"
using namespace std;

#include "cutstockCfg.h"

namespace problemSpecific{
/*                                        INPUT/OUTPUT ROUTINES                              */
int instanceType;
#define BP1_FILE 1
#define CLASSICAL 2
int instNo;                       //the total number of instances
int maxInstNo;                    //the maximum number of instance, as read from the cfg file
int currInst;                     //the instance to be solved
fstream inStream;                 //input stream
char* inputFile=NULL;             //Using char* for compatibility with fstream
string instNameFromFile;   

//*Problem-independent* data: we always deal with capacitated problems having
int n;                    //n items/arc/etc
double C;                 //capacity
double * w=NULL;          //weight per item/arc/etc
double * b=NULL;          //required number of services/items
double baseCost = 1;      //minimum pattern cost when (base) capacity is respected

//This function returns the file format of the cspFile: BP1_FILE or CLASSICAL
int cspFileFormat(char*cspFile){
    int pointPos = strlen(cspFile)-4;
    if(cspFile[pointPos]!='.'||//no extension
      (cspFile[pointPos+1]=='d'&&cspFile[pointPos+2]=='a'&&cspFile[pointPos+3]=='t')//.dat
    ){
            clog<<"Reading a classical file (no 3-character extension):"<<cspFile<<".\n";
            return CLASSICAL;
    }
    if(cspFile[pointPos+1]=='1'&&cspFile[pointPos+2]=='b'&&cspFile[pointPos+3]=='p'){//.1bp
            clog<<"Reading a 1bp file:"<<cspFile<<".\n";
            return BP1_FILE;
    }
    cerr<<"Unknown csp file format (extension '"<<cspFile+pointPos+1<<"') not recognized\n"; 
    exit(EXIT_FAILURE);
}

void readNextInst(){
    int dump;
    if(instanceType==BP1_FILE){
        inStream >> C;
        inStream >> dump;
        inStream >> n;
        clog<<"====================================\n READING 1BP instance ID="<<currInst<<endl;;
        cout<<setw(20)<<currInst;
    }else{
        inStream >> n;
        inStream >> C;
        clog<<"====================================\n READING classical instance";
        cout<<setw(20)<<instNameFromFile;
    }
    clog<<" with n="<<n<<" and tot. cap="<<C<<"\n=========================================\n"<<endl;

    w = new double[n];
    b = new double[n];

    for(int i=0;i<n;i++){ 
      inStream >> w[i];
      inStream >> b[i];
    }
    
    currInst++;

    #ifdef SCALED_INSTANCE
    cout<<"$^*$";
    C = C * 1000;
    for(int i=0;i<n;i++)
        w[i] = w[i]*1000 - i%10;
    #endif
    cout<<"&";
}

//Return: 1 if instances are still in the file,  [instNameFromFile for classical instances)]
int moreInst(){
    if(instanceType==BP1_FILE)  
        return currInst<=instNo;
    else{
        instNameFromFile = ""; 
        while(instNameFromFile.size()==0&&inStream.good()){
            getline(inStream,instNameFromFile);
        } 
        return inStream.good();
    }
}
//Prepare the data to be able to load a new input file 
//Input : filename
//Return: stream inStream, int currInst, int instanceType, int instNo (nr of instance)
void prepareNewInput(char*filename){
    if(inputFile!=NULL)
        delete[] inputFile;

    inputFile = new char[strlen(filename)+1];
    strcpy(inputFile,filename);
    inStream.open(inputFile, fstream::in);  
    currInst = 1;                             //first instance to be loaded 
    instanceType = cspFileFormat(inputFile);
    if(instanceType == BP1_FILE)
        inStream >> instNo;                   //number of instances
    else                                      //unknown number of instances, but the read stops when
        instNo = -1;                          //there are no other instances names in the file
}
//return value: 0 when there is nothing more to load, 1 otherwise
//return implicit: number of variables n,
//                 lower bound on each variable l
//                 objective function coefficients b
//Other problem specific data is loaded in: w and C
int loadData(char *filename, int &n, double* &l, double* &u, double* &b){
    if(inputFile==NULL||                         //first call ever
       strcmp(inputFile,filename) )              //the input file changed
            prepareNewInput(filename);
    if(!moreInst()){
        clog<<"No more instances\n";
        return 0;
    }
    if(b!=NULL){
        delete[] l;
        delete[] u;
        delete[] b;
        delete[] w;
    }
    l=NULL;w=NULL; b=NULL;
    readNextInst();    
    n = problemSpecific::n;
    b = problemSpecific::b;
    l = new double[n];
    u = new double[n];
    //trivial bounds and constraints (the ub one is added by cplex example as well for col gen)
    for(int i=0;i<n;i++){
        l[i] = 0; 
        int maxTimesUniqueItem = floor(C/w[i]);
        u[i] = min(b[i],(double)1/maxTimesUniqueItem);
    }
    return 1;                                  
}

int loadData(char *filename, int &n, double* &l, double* &u, double* &b, double &lBndBase){
    lBndBase = 0;
    return loadData(filename,n,l,u,b);
}
/*                                       RAY SUB PROBLEM                                          */
double cBest;                                  //Global variables to store the best solution
int    pBest;                                  //tBest = cBest/pBest
int *  aBest;
typedef struct state {                          //main DP state
    int p;
    double w;
    int step;                                   //discovery step/moment in dyn prog
    int    itIdx;
    struct state* next;
    struct state* prec;                         //precedence relation
    int newStRoot;                              //step when it becomes root for edges to other states
    state(int p_, double w_,int step_  ,int itIdx_   ,struct state*next_,struct state*prec_):
          p(p_) , w(w_)    ,step(step_),itIdx(itIdx_),next(next_)       ,prec(prec_),newStRoot(0)
          {};
} stateList;

//Finds the best state and fills in the variables: aBest,cBest,tBest,pBest
//Input: next after head, assuming it to be not-null
void findBestState(stateList* s1){
    if(s1==NULL){
        cerr<<"Error: no states found, is the instance really correct?"<<endl;
        assert(s1!=NULL);
    }

    double cNow;
    int pNow;
    cBest = EXT_F(s1->w/C);
    pBest = s1->p;
    stateList * bestState = s1;
    s1    = s1->next; 
    while(s1!=NULL){                   //1. Go through all states and find best one
        cNow = EXT_F(s1->w/C); 
        pNow = s1->p;
        if(cNow*pBest<cBest*pNow){
            cBest     = cNow;
            pBest     = pNow;
            bestState = s1;
        }
        s1 = s1->next;
    }
    s1 = bestState;
    cBest = EXT_F(s1->w/C);
    pBest = s1->p;
    int prvProfBest ;
    while(s1->p!=0){                   //2. Follow precedence relation to construct a
        aBest[s1->itIdx]++;
        prvProfBest = s1->p;
        s1 = s1->prec;
        assert(prvProfBest > s1->p);
    }
}

//insert state after s2Prv and returns the new state
stateList* insertAfter(int p, double w, int step, int i, stateList* s2Prv, stateList*s1){
    stateList*sNew = new stateList(p,w,step,i, s2Prv->next, s1);
    s2Prv->next = sNew;
    return sNew;
}

//erases all states from s1 inclusive
void eraseAllStates(stateList &head){
    int count = 0;
    while(head.next!=NULL){
        stateList * toDel = head.next;       
        head.next = head.next->next;
        count++;
        delete toDel;
    }
    clog<<"Number of deleted states:"<<count<<endl;
}
//prints all states given state head
void printStates(stateList &head, int step){
      clog<<"========================================\nStates after "<<step<<" steps\n";
      stateList* s1 = &head;
      do{
         clog<<s1->p<<":"<<s1->w<<",step="<<s1->step;
         if(s1->prec!=NULL)
              clog<<"(prec="<<s1->prec->p<<")\n";
         else
              clog<<endl; 
         s1=s1->next;
      }while (s1!=NULL);
      clog<<endl;
}

//#define USE_MINKNAP
#ifdef USE_MINKNAP
#include "minknapIrmTest.cpp"
#endif
//The main algorithm: the result will be put in the last three arguments
//Attention: all r values must be >=0
void irmSubproblem(int * r, double &t, int * &a, double &ca, int& p){
#ifdef USE_MINKNAP
    irmMinknap(r,t,a,ca,p);
    return ;
#endif
    int pNew,newStateRequest = 0;
    stateList head(0,0,0,-1,NULL,NULL);                          //profit-ordered list
    stateList *s1, *s2;                                          //first and second iterator
    stateList *s1Prv=NULL, *s2Prv=NULL;                          //state before second iterator
    //Dominated states are not erased instantly, but put in a list for precedence relations that can
    //be useful to re-construct a in the end;
    stateList sDomHead(0,0,0,-1,NULL,NULL);
    stateList *sDomTail=&sDomHead;
    int step = 0;                        
    for (int i=0;i<n;i++){
        for (int j=1;j<=b[i];j++){
            if(j*w[i]>EXT_MAX*C)                                 //do not pile up too much
                break;
            s1 =&head;                                           
            do{                                                  //go through states
                pNew = s1->p+r[i];
                if( (s1->w+w[i]<=EXT_MAX*C) &&   
                    (s1->step<=step) &&                          //ignore creations of current loop
                    (j==1||s1->step==step)                       //if j>1, ignore creations of j-1
                  ){
                     s2 = s1;
                     newStateRequest = false;
                     do{
                            //cout<<"hit"<<step<<","<<s1->step<<"\n";
                            s2Prv = s2;
                            s2 = s2->next;
                            if(s2==NULL){
                                   newStateRequest = true;
                                   break;
                            }
                            if(s2->p > pNew){
                                if(s2->w > s1->w+w[i])            //the state is *only* inserted if
                                   newStateRequest = true;        //it has a smaller weight then
                                break;                            //the next state (of higher prft)
                            }                                     
                    }while (true);                                //all above ifs/breaks show stop conds
                    if(newStateRequest){
                        assert(s2Prv->p<=pNew);
                        if( (s2Prv->p < pNew)
                          ||(s2Prv->w > s1->w+w[i])){             //improving s2Prv with p=pNew
                                s1->newStRoot = step;             //s1 becomes root for a new edge
                                insertAfter(pNew,s1->w+w[i],step+1,i,s2Prv,s1);
                        }
                    } 
                }//finished discovery for current i/j and s1 (start point)
                if((s1->next!=NULL)&&(s1->next->w < s1->w)){     //if s1->next dominates s1
                    s1Prv->next = s1->next;                      //remove s1 from the main list
                    if(s1->newStRoot>0){                         //if s1 generates some states
                        sDomTail->next = s1;                     //do not forget refs to it
                        sDomTail = s1;
                        s1->next=NULL;
                    }else                                        //CHECK: this can be useless
                        delete s1;
                    s1 = s1Prv;
                } 
                s1Prv = s1;
                s1=s1->next;
            }while(s1!=NULL);
            step++;
        }//end for j, 
    }//end for i
    aBest = new int[n]; 
    for (int i=0;i<n;i++)
        aBest[i]=0;
    findBestState(head.next);
    a = aBest, ca = cBest, p=pBest;
    t =ca/p;
    eraseAllStates(head);       
    //printStates(sDomHead, -3);
    eraseAllStates(sDomHead);       

    //
    double wTot = 0;
    for(int i=0;i<n;i++){
       if(a[i]!=0){
            clog<<"a["<<i<<"]="<<a[i]<<":[w r="<<w[i]<<" "<<r[i]<<"],";
        }
       wTot +=a[i]*w[i];
    }
    clog<<"\nwTotal="<<wTot<<endl;
    clog<<"t="<<t<<endl;
    clog<<"p="<<pBest<<endl;
    clog<<"ca="<<ca<<endl;
}
}//end namespace problemSpecific

